# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CGNS/NAV/T/Q7LogWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Q7LogWindow(object):
    def setupUi(self, Q7LogWindow):
        Q7LogWindow.setObjectName("Q7LogWindow")
        Q7LogWindow.resize(600, 400)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Q7LogWindow.sizePolicy().hasHeightForWidth())
        Q7LogWindow.setSizePolicy(sizePolicy)
        Q7LogWindow.setMinimumSize(QtCore.QSize(500, 140))
        Q7LogWindow.setMaximumSize(QtCore.QSize(16777215, 16777215))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/icons/cgSpy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Q7LogWindow.setWindowIcon(icon)
        self.verticalLayout = QtWidgets.QVBoxLayout(Q7LogWindow)
        self.verticalLayout.setObjectName("verticalLayout")
        self.eLog = Q7PythonEditor(Q7LogWindow)
        self.eLog.setMinimumSize(QtCore.QSize(0, 0))
        self.eLog.setFrameShadow(QtWidgets.QFrame.Raised)
        self.eLog.setLineWidth(0)
        self.eLog.setObjectName("eLog")
        self.verticalLayout.addWidget(self.eLog)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetNoConstraint)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(868, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.bClear = QtWidgets.QPushButton(Q7LogWindow)
        self.bClear.setObjectName("bClear")
        self.horizontalLayout_2.addWidget(self.bClear)
        self.bClose = QtWidgets.QPushButton(Q7LogWindow)
        self.bClose.setObjectName("bClose")
        self.horizontalLayout_2.addWidget(self.bClose)
        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(Q7LogWindow)
        QtCore.QMetaObject.connectSlotsByName(Q7LogWindow)

    def retranslateUi(self, Q7LogWindow):
        _translate = QtCore.QCoreApplication.translate
        Q7LogWindow.setWindowTitle(_translate("Q7LogWindow", "Dialog"))
        self.bClear.setText(_translate("Q7LogWindow", "Clear"))
        self.bClose.setText(_translate("Q7LogWindow", "Hide"))
from CGNS.NAV.weditors import Q7PythonEditor
from . import Res_rc



#  -------------------------------------------------------------------------
#  pyCGNS.NAV - Python package for CFD General Notation System - NAVigater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#

import numpy

import CGNS.NAV.mtree as NMT
from CGNS.NAV.mtree import Q7TreeView, Q7TreeItem, Q7TreeModel
from CGNS.NAV.moption import Q7OptionContext as OCTXT


## -----------------------------------------------------------------
# class Q7MergeTreeView(Q7MergeView):
#    def  __init__(self,parent):
#        Q7TreeView.__init__(self,parent)

# -----------------------------------------------------------------
class Q7TreeMergeModel(Q7TreeModel):
    def __init__(self, fgprint, parent=None):
        super(Q7TreeMergeModel, self).__init__(fgprint, parent)
        self.addA = True
        self.addB = True

    def itemNew(self, fg, nt, md, tg, pi):
        return Q7MergeItem(fg, nt, md, tg, pi)


TAG_FRONT = '???'
TAG_BACK = '&'  # single char required
TAG_LENGTH = 5


# -----------------------------------------------------------------
class Q7MergeItem(Q7TreeItem):
    def __init__(self, fgprint, data, model, tag="", parent=None):
        super(Q7MergeItem, self).__init__(fgprint, data, model, tag, parent)
        self._model = model

    def columnCount(self):
        return NMT.COLUMN_LAST + 1

    def notagName(self):
        name = self.sidsName()
        if ((len(name) > TAG_LENGTH) and
                (name[-1] == TAG_BACK) and
                (name[-TAG_LENGTH:-len(TAG_BACK) - 1] == TAG_FRONT)):
            return name[:-TAG_LENGTH]
        return name

    def data(self, column):
        if not self._model.addA and self.userState() == NMT.STUSR_A:
            print('NO A')
            return None
        if not self._model.addB and self.userState() == NMT.STUSR_B:
            print('NO B')
        if self._itemnode is None:
            return self._title[column]
        if column == NMT.COLUMN_NAME:
            return self.notagName()
        if column == NMT.COLUMN_SIDS:
            return self.sidsType()
        if column == NMT.COLUMN_FLAG_LINK:
            return self.sidsLinkStatus()
        if column == NMT.COLUMN_SHAPE:
            if self.sidsValue() is None:
                return None
            if not isinstance(self.sidsValue(), numpy.ndarray):
                return None
            return str(self.sidsValue().shape)
        if column == NMT.COLUMN_FLAG_SELECT:
            return self._states['mark']
        if column == NMT.COLUMN_DATATYPE:
            return self.sidsDataType()
        if column == NMT.COLUMN_FLAG_CHECK:
            return self._states['check']
        if column == NMT.COLUMN_FLAG_USER:
            return self._states['user']
        if column == NMT.COLUMN_VALUE:
            if self._lazy:
                return NMT.LAZYVALUE
            if self.sidsValue() is None:
                return None
            if isinstance(self.sidsValue(), numpy.ndarray):
                vsize = 1
                for x in self.sidsValue().shape:
                    vsize *= x
                if ((vsize > OCTXT.MaxDisplayDataSize) and
                        (OCTXT.MaxDisplayDataSize > 0)):
                    return NMT.HIDEVALUE
                if self.sidsValue().dtype.char in ['S', 'c']:
                    if len(self.sidsValue().shape) == 1:
                        return self.sidsValue().tostring().decode('ascii')
                    if len(self.sidsValue().shape) > 2:
                        return NMT.HIDEVALUE
                    return NMT.HIDEVALUE
                if (self.sidsValue().shape == (1,)) and OCTXT.Show1DAsPlain:
                    return str(self.sidsValue()[0])
            return str(self.sidsValue().tolist())[:100]
        return None

# -----------------------------------------------------------------



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CGNS/NAV/T/Q7TreeWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Q7TreeWindow(object):
    def setupUi(self, Q7TreeWindow):
        Q7TreeWindow.setObjectName("Q7TreeWindow")
        Q7TreeWindow.resize(1124, 300)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/icons/cgSpy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Q7TreeWindow.setWindowIcon(icon)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Q7TreeWindow)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.bSave = QtWidgets.QToolButton(Q7TreeWindow)
        self.bSave.setMinimumSize(QtCore.QSize(25, 25))
        self.bSave.setMaximumSize(QtCore.QSize(25, 25))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/icons/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSave.setIcon(icon1)
        self.bSave.setObjectName("bSave")
        self.horizontalLayout.addWidget(self.bSave)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.bSaveAs = QtWidgets.QToolButton(Q7TreeWindow)
        self.bSaveAs.setMinimumSize(QtCore.QSize(25, 25))
        self.bSaveAs.setMaximumSize(QtCore.QSize(25, 25))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/icons/tree-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSaveAs.setIcon(icon2)
        self.bSaveAs.setObjectName("bSaveAs")
        self.horizontalLayout.addWidget(self.bSaveAs)
        self.bPatternDB = QtWidgets.QToolButton(Q7TreeWindow)
        self.bPatternDB.setMinimumSize(QtCore.QSize(25, 25))
        self.bPatternDB.setMaximumSize(QtCore.QSize(25, 25))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/icons/pattern-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bPatternDB.setIcon(icon3)
        self.bPatternDB.setObjectName("bPatternDB")
        self.horizontalLayout.addWidget(self.bPatternDB)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.bZoomOut = QtWidgets.QToolButton(Q7TreeWindow)
        self.bZoomOut.setMinimumSize(QtCore.QSize(25, 25))
        self.bZoomOut.setMaximumSize(QtCore.QSize(25, 25))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/icons/level-out.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bZoomOut.setIcon(icon4)
        self.bZoomOut.setObjectName("bZoomOut")
        self.horizontalLayout.addWidget(self.bZoomOut)
        self.bZoomAll = QtWidgets.QPushButton(Q7TreeWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bZoomAll.sizePolicy().hasHeightForWidth())
        self.bZoomAll.setSizePolicy(sizePolicy)
        self.bZoomAll.setMinimumSize(QtCore.QSize(25, 25))
        self.bZoomAll.setMaximumSize(QtCore.QSize(25, 25))
        self.bZoomAll.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/icons/level-all.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bZoomAll.setIcon(icon5)
        self.bZoomAll.setObjectName("bZoomAll")
        self.horizontalLayout.addWidget(self.bZoomAll)
        self.bZoomIn = QtWidgets.QToolButton(Q7TreeWindow)
        self.bZoomIn.setMinimumSize(QtCore.QSize(25, 25))
        self.bZoomIn.setMaximumSize(QtCore.QSize(25, 25))
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/images/icons/level-in.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bZoomIn.setIcon(icon6)
        self.bZoomIn.setObjectName("bZoomIn")
        self.horizontalLayout.addWidget(self.bZoomIn)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.bMarkAll = QtWidgets.QToolButton(Q7TreeWindow)
        self.bMarkAll.setMinimumSize(QtCore.QSize(25, 25))
        self.bMarkAll.setMaximumSize(QtCore.QSize(25, 25))
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/images/icons/flag-all.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bMarkAll.setIcon(icon7)
        self.bMarkAll.setObjectName("bMarkAll")
        self.horizontalLayout.addWidget(self.bMarkAll)
        self.bSwapMarks = QtWidgets.QToolButton(Q7TreeWindow)
        self.bSwapMarks.setMinimumSize(QtCore.QSize(25, 25))
        self.bSwapMarks.setMaximumSize(QtCore.QSize(25, 25))
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/images/icons/flag-revert.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSwapMarks.setIcon(icon8)
        self.bSwapMarks.setObjectName("bSwapMarks")
        self.horizontalLayout.addWidget(self.bSwapMarks)
        self.bUnmarkAll_2 = QtWidgets.QPushButton(Q7TreeWindow)
        self.bUnmarkAll_2.setMinimumSize(QtCore.QSize(25, 25))
        self.bUnmarkAll_2.setMaximumSize(QtCore.QSize(25, 25))
        self.bUnmarkAll_2.setText("")
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/images/icons/flag-none.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bUnmarkAll_2.setIcon(icon9)
        self.bUnmarkAll_2.setObjectName("bUnmarkAll_2")
        self.horizontalLayout.addWidget(self.bUnmarkAll_2)
        self.bMarksAsList = QtWidgets.QToolButton(Q7TreeWindow)
        self.bMarksAsList.setMinimumSize(QtCore.QSize(25, 25))
        self.bMarksAsList.setMaximumSize(QtCore.QSize(25, 25))
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/images/icons/operate-list.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bMarksAsList.setIcon(icon10)
        self.bMarksAsList.setObjectName("bMarksAsList")
        self.horizontalLayout.addWidget(self.bMarksAsList)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.bCheck = QtWidgets.QToolButton(Q7TreeWindow)
        self.bCheck.setMinimumSize(QtCore.QSize(25, 25))
        self.bCheck.setMaximumSize(QtCore.QSize(25, 25))
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/images/icons/check-all.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bCheck.setIcon(icon11)
        self.bCheck.setObjectName("bCheck")
        self.horizontalLayout.addWidget(self.bCheck)
        self.bClearChecks = QtWidgets.QToolButton(Q7TreeWindow)
        self.bClearChecks.setMinimumSize(QtCore.QSize(25, 25))
        self.bClearChecks.setMaximumSize(QtCore.QSize(25, 25))
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/images/icons/check-clear.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bClearChecks.setIcon(icon12)
        self.bClearChecks.setObjectName("bClearChecks")
        self.horizontalLayout.addWidget(self.bClearChecks)
        self.bCheckList = QtWidgets.QToolButton(Q7TreeWindow)
        self.bCheckList.setMinimumSize(QtCore.QSize(25, 25))
        self.bCheckList.setMaximumSize(QtCore.QSize(25, 25))
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/images/icons/check-list.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bCheckList.setIcon(icon13)
        self.bCheckList.setObjectName("bCheckList")
        self.horizontalLayout.addWidget(self.bCheckList)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.bSelectLinkSrc = QtWidgets.QPushButton(Q7TreeWindow)
        self.bSelectLinkSrc.setMinimumSize(QtCore.QSize(25, 25))
        self.bSelectLinkSrc.setMaximumSize(QtCore.QSize(25, 25))
        self.bSelectLinkSrc.setText("")
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(":/images/icons/link-src.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSelectLinkSrc.setIcon(icon14)
        self.bSelectLinkSrc.setCheckable(True)
        self.bSelectLinkSrc.setObjectName("bSelectLinkSrc")
        self.horizontalLayout.addWidget(self.bSelectLinkSrc)
        self.bSelectLinkDst = QtWidgets.QPushButton(Q7TreeWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(25)
        sizePolicy.setVerticalStretch(25)
        sizePolicy.setHeightForWidth(self.bSelectLinkDst.sizePolicy().hasHeightForWidth())
        self.bSelectLinkDst.setSizePolicy(sizePolicy)
        self.bSelectLinkDst.setMinimumSize(QtCore.QSize(25, 25))
        self.bSelectLinkDst.setMaximumSize(QtCore.QSize(25, 25))
        self.bSelectLinkDst.setText("")
        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap(":/images/icons/link-dst.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSelectLinkDst.setIcon(icon15)
        self.bSelectLinkDst.setCheckable(True)
        self.bSelectLinkDst.setObjectName("bSelectLinkDst")
        self.horizontalLayout.addWidget(self.bSelectLinkDst)
        self.bAddLink = QtWidgets.QPushButton(Q7TreeWindow)
        self.bAddLink.setMinimumSize(QtCore.QSize(25, 25))
        self.bAddLink.setMaximumSize(QtCore.QSize(25, 25))
        self.bAddLink.setText("")
        icon16 = QtGui.QIcon()
        icon16.addPixmap(QtGui.QPixmap(":/images/icons/link-add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bAddLink.setIcon(icon16)
        self.bAddLink.setObjectName("bAddLink")
        self.horizontalLayout.addWidget(self.bAddLink)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem5)
        self.bToolsView = QtWidgets.QPushButton(Q7TreeWindow)
        self.bToolsView.setMinimumSize(QtCore.QSize(25, 25))
        self.bToolsView.setMaximumSize(QtCore.QSize(25, 25))
        self.bToolsView.setText("")
        icon17 = QtGui.QIcon()
        icon17.addPixmap(QtGui.QPixmap(":/images/icons/tools.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bToolsView.setIcon(icon17)
        self.bToolsView.setObjectName("bToolsView")
        self.horizontalLayout.addWidget(self.bToolsView)
        self.bFormView = QtWidgets.QPushButton(Q7TreeWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bFormView.sizePolicy().hasHeightForWidth())
        self.bFormView.setSizePolicy(sizePolicy)
        self.bFormView.setMinimumSize(QtCore.QSize(25, 25))
        self.bFormView.setMaximumSize(QtCore.QSize(25, 25))
        self.bFormView.setText("")
        icon18 = QtGui.QIcon()
        icon18.addPixmap(QtGui.QPixmap(":/images/icons/form-open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bFormView.setIcon(icon18)
        self.bFormView.setObjectName("bFormView")
        self.horizontalLayout.addWidget(self.bFormView)
        self.bVTKView = QtWidgets.QToolButton(Q7TreeWindow)
        self.bVTKView.setMinimumSize(QtCore.QSize(25, 25))
        self.bVTKView.setMaximumSize(QtCore.QSize(25, 25))
        icon19 = QtGui.QIcon()
        icon19.addPixmap(QtGui.QPixmap(":/images/icons/vtk.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bVTKView.setIcon(icon19)
        self.bVTKView.setObjectName("bVTKView")
        self.horizontalLayout.addWidget(self.bVTKView)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem6)
        self.bPatternView = QtWidgets.QPushButton(Q7TreeWindow)
        self.bPatternView.setMinimumSize(QtCore.QSize(25, 25))
        self.bPatternView.setMaximumSize(QtCore.QSize(25, 25))
        self.bPatternView.setText("")
        icon20 = QtGui.QIcon()
        icon20.addPixmap(QtGui.QPixmap(":/images/icons/pattern-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bPatternView.setIcon(icon20)
        self.bPatternView.setObjectName("bPatternView")
        self.horizontalLayout.addWidget(self.bPatternView)
        self.bLinkView = QtWidgets.QToolButton(Q7TreeWindow)
        self.bLinkView.setMinimumSize(QtCore.QSize(25, 25))
        self.bLinkView.setMaximumSize(QtCore.QSize(25, 25))
        icon21 = QtGui.QIcon()
        icon21.addPixmap(QtGui.QPixmap(":/images/icons/link-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bLinkView.setIcon(icon21)
        self.bLinkView.setObjectName("bLinkView")
        self.horizontalLayout.addWidget(self.bLinkView)
        self.bQueryView = QtWidgets.QToolButton(Q7TreeWindow)
        self.bQueryView.setMinimumSize(QtCore.QSize(25, 25))
        self.bQueryView.setMaximumSize(QtCore.QSize(25, 25))
        icon22 = QtGui.QIcon()
        icon22.addPixmap(QtGui.QPixmap(":/images/icons/operate-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bQueryView.setIcon(icon22)
        self.bQueryView.setObjectName("bQueryView")
        self.horizontalLayout.addWidget(self.bQueryView)
        self.bCheckView = QtWidgets.QToolButton(Q7TreeWindow)
        self.bCheckView.setMinimumSize(QtCore.QSize(25, 25))
        self.bCheckView.setMaximumSize(QtCore.QSize(25, 25))
        icon23 = QtGui.QIcon()
        icon23.addPixmap(QtGui.QPixmap(":/images/icons/check-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bCheckView.setIcon(icon23)
        self.bCheckView.setObjectName("bCheckView")
        self.horizontalLayout.addWidget(self.bCheckView)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem7)
        self.bScreenShot = QtWidgets.QToolButton(Q7TreeWindow)
        self.bScreenShot.setMinimumSize(QtCore.QSize(25, 25))
        self.bScreenShot.setMaximumSize(QtCore.QSize(25, 25))
        icon24 = QtGui.QIcon()
        icon24.addPixmap(QtGui.QPixmap(":/images/icons/snapshot.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bScreenShot.setIcon(icon24)
        self.bScreenShot.setObjectName("bScreenShot")
        self.horizontalLayout.addWidget(self.bScreenShot)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.treeview = Q7TreeView(Q7TreeWindow)
        self.treeview.viewport().setProperty("cursor", QtGui.QCursor(QtCore.Qt.CrossCursor))
        self.treeview.setAutoScroll(False)
        self.treeview.setProperty("showDropIndicator", False)
        self.treeview.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollPerItem)
        self.treeview.setIndentation(16)
        self.treeview.setRootIsDecorated(True)
        self.treeview.setUniformRowHeights(True)
        self.treeview.setExpandsOnDoubleClick(False)
        self.treeview.setObjectName("treeview")
        self.horizontalLayout_2.addWidget(self.treeview)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.bBackControl = QtWidgets.QPushButton(Q7TreeWindow)
        self.bBackControl.setMinimumSize(QtCore.QSize(25, 25))
        self.bBackControl.setMaximumSize(QtCore.QSize(25, 25))
        self.bBackControl.setText("")
        icon25 = QtGui.QIcon()
        icon25.addPixmap(QtGui.QPixmap(":/images/icons/top.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bBackControl.setIcon(icon25)
        self.bBackControl.setObjectName("bBackControl")
        self.horizontalLayout_3.addWidget(self.bBackControl)
        self.bInfo = QtWidgets.QPushButton(Q7TreeWindow)
        self.bInfo.setMinimumSize(QtCore.QSize(25, 25))
        self.bInfo.setMaximumSize(QtCore.QSize(25, 25))
        self.bInfo.setText("")
        icon26 = QtGui.QIcon()
        icon26.addPixmap(QtGui.QPixmap(":/images/icons/help-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bInfo.setIcon(icon26)
        self.bInfo.setObjectName("bInfo")
        self.horizontalLayout_3.addWidget(self.bInfo)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem8)
        self.lineEditLock = QtWidgets.QPushButton(Q7TreeWindow)
        self.lineEditLock.setMinimumSize(QtCore.QSize(25, 25))
        self.lineEditLock.setMaximumSize(QtCore.QSize(25, 25))
        self.lineEditLock.setText("")
        icon27 = QtGui.QIcon()
        icon27.addPixmap(QtGui.QPixmap(":/images/icons/optional-sids-node.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon27.addPixmap(QtGui.QPixmap(":/images/icons/lock-scroll.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.lineEditLock.setIcon(icon27)
        self.lineEditLock.setCheckable(True)
        self.lineEditLock.setObjectName("lineEditLock")
        self.horizontalLayout_3.addWidget(self.lineEditLock)
        self.lineEdit = QtWidgets.QLineEdit(Q7TreeWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit.sizePolicy().hasHeightForWidth())
        self.lineEdit.setSizePolicy(sizePolicy)
        self.lineEdit.setMinimumSize(QtCore.QSize(0, 0))
        self.lineEdit.setObjectName("lineEdit")
        self.horizontalLayout_3.addWidget(self.lineEdit)
        self.bPreviousMark = QtWidgets.QToolButton(Q7TreeWindow)
        self.bPreviousMark.setMinimumSize(QtCore.QSize(25, 25))
        self.bPreviousMark.setMaximumSize(QtCore.QSize(25, 25))
        icon28 = QtGui.QIcon()
        icon28.addPixmap(QtGui.QPixmap(":/images/icons/node-sids-opened.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bPreviousMark.setIcon(icon28)
        self.bPreviousMark.setObjectName("bPreviousMark")
        self.horizontalLayout_3.addWidget(self.bPreviousMark)
        self.bUnmarkAll_1 = QtWidgets.QToolButton(Q7TreeWindow)
        self.bUnmarkAll_1.setMinimumSize(QtCore.QSize(25, 25))
        self.bUnmarkAll_1.setMaximumSize(QtCore.QSize(25, 25))
        icon29 = QtGui.QIcon()
        icon29.addPixmap(QtGui.QPixmap(":/images/icons/node-sids-leaf.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bUnmarkAll_1.setIcon(icon29)
        self.bUnmarkAll_1.setObjectName("bUnmarkAll_1")
        self.horizontalLayout_3.addWidget(self.bUnmarkAll_1)
        self.bNextMark = QtWidgets.QToolButton(Q7TreeWindow)
        self.bNextMark.setMinimumSize(QtCore.QSize(25, 25))
        self.bNextMark.setMaximumSize(QtCore.QSize(25, 25))
        icon30 = QtGui.QIcon()
        icon30.addPixmap(QtGui.QPixmap(":/images/icons/node-sids-closed.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bNextMark.setIcon(icon30)
        self.bNextMark.setObjectName("bNextMark")
        self.horizontalLayout_3.addWidget(self.bNextMark)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.retranslateUi(Q7TreeWindow)
        QtCore.QMetaObject.connectSlotsByName(Q7TreeWindow)

    def retranslateUi(self, Q7TreeWindow):
        _translate = QtCore.QCoreApplication.translate
        Q7TreeWindow.setWindowTitle(_translate("Q7TreeWindow", "Form"))
        self.bSave.setToolTip(_translate("Q7TreeWindow", "Save tree (overwrite existing file)"))
        self.bSave.setText(_translate("Q7TreeWindow", "..."))
        self.bSaveAs.setToolTip(_translate("Q7TreeWindow", "Save tree as (creates a new file)"))
        self.bSaveAs.setText(_translate("Q7TreeWindow", "..."))
        self.bPatternDB.setToolTip(_translate("Q7TreeWindow", "Save tree as profile"))
        self.bPatternDB.setText(_translate("Q7TreeWindow", "..."))
        self.bZoomOut.setToolTip(_translate("Q7TreeWindow", "Collapse lowest tree level"))
        self.bZoomOut.setText(_translate("Q7TreeWindow", "..."))
        self.bZoomAll.setToolTip(_translate("Q7TreeWindow", "Expand all tree"))
        self.bZoomIn.setToolTip(_translate("Q7TreeWindow", "Expand lowest tree level"))
        self.bZoomIn.setText(_translate("Q7TreeWindow", "..."))
        self.bMarkAll.setToolTip(_translate("Q7TreeWindow", "Mark all nodes"))
        self.bMarkAll.setText(_translate("Q7TreeWindow", "..."))
        self.bSwapMarks.setToolTip(_translate("Q7TreeWindow", "Swap marked/unmarked nodes"))
        self.bSwapMarks.setText(_translate("Q7TreeWindow", "..."))
        self.bMarksAsList.setToolTip(_translate("Q7TreeWindow", "Open selected nodes list"))
        self.bMarksAsList.setText(_translate("Q7TreeWindow", "..."))
        self.bCheck.setToolTip(_translate("Q7TreeWindow", "Check selected nodes"))
        self.bCheck.setText(_translate("Q7TreeWindow", "..."))
        self.bClearChecks.setToolTip(_translate("Q7TreeWindow", "Remove check labels"))
        self.bClearChecks.setText(_translate("Q7TreeWindow", "..."))
        self.bCheckList.setToolTip(_translate("Q7TreeWindow", "Open diagnostics list"))
        self.bCheckList.setText(_translate("Q7TreeWindow", "..."))
        self.bSelectLinkSrc.setToolTip(_translate("Q7TreeWindow", "Add selected node as parent to link source"))
        self.bSelectLinkDst.setToolTip(_translate("Q7TreeWindow", "Add selected node as link destination"))
        self.bAddLink.setToolTip(_translate("Q7TreeWindow", "Create a link using selected link source/destination"))
        self.bToolsView.setToolTip(_translate("Q7TreeWindow", "Open Tools view"))
        self.bFormView.setToolTip(_translate("Q7TreeWindow", "Open Form view on selected node"))
        self.bVTKView.setToolTip(_translate("Q7TreeWindow", "Open VTK view"))
        self.bVTKView.setText(_translate("Q7TreeWindow", "..."))
        self.bPatternView.setToolTip(_translate("Q7TreeWindow", "Open Patterns view"))
        self.bLinkView.setToolTip(_translate("Q7TreeWindow", "Open Links view"))
        self.bLinkView.setText(_translate("Q7TreeWindow", "..."))
        self.bQueryView.setToolTip(_translate("Q7TreeWindow", "Open Queries view"))
        self.bQueryView.setText(_translate("Q7TreeWindow", "..."))
        self.bCheckView.setToolTip(_translate("Q7TreeWindow", "Open Check view"))
        self.bCheckView.setText(_translate("Q7TreeWindow", "..."))
        self.bScreenShot.setToolTip(_translate("Q7TreeWindow", "Save tree view snapshot"))
        self.bScreenShot.setText(_translate("Q7TreeWindow", "..."))
        self.bBackControl.setToolTip(_translate("Q7TreeWindow", "Raise Control window"))
        self.lineEditLock.setToolTip(_translate("Q7TreeWindow", "Toggle tree selection updates the edit line"))
        self.bPreviousMark.setToolTip(_translate("Q7TreeWindow", "Select previous marked node"))
        self.bPreviousMark.setText(_translate("Q7TreeWindow", "..."))
        self.bUnmarkAll_1.setToolTip(_translate("Q7TreeWindow", "Unmark all nodes"))
        self.bUnmarkAll_1.setText(_translate("Q7TreeWindow", "..."))
        self.bNextMark.setToolTip(_translate("Q7TreeWindow", "Select next marked node"))
        self.bNextMark.setText(_translate("Q7TreeWindow", "..."))
from CGNS.NAV.mtree import Q7TreeView
from . import Res_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CGNS/NAV/T/Q7DiagWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Q7DiagWindow(object):
    def setupUi(self, Q7DiagWindow):
        Q7DiagWindow.setObjectName("Q7DiagWindow")
        Q7DiagWindow.resize(715, 375)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Q7DiagWindow.sizePolicy().hasHeightForWidth())
        Q7DiagWindow.setSizePolicy(sizePolicy)
        Q7DiagWindow.setMinimumSize(QtCore.QSize(715, 350))
        Q7DiagWindow.setMaximumSize(QtCore.QSize(1200, 900))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/icons/cgSpy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Q7DiagWindow.setWindowIcon(icon)
        self.gridLayout = QtWidgets.QGridLayout(Q7DiagWindow)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.bBackControl = QtWidgets.QPushButton(Q7DiagWindow)
        self.bBackControl.setMinimumSize(QtCore.QSize(25, 25))
        self.bBackControl.setMaximumSize(QtCore.QSize(25, 25))
        self.bBackControl.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/icons/top.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bBackControl.setIcon(icon1)
        self.bBackControl.setObjectName("bBackControl")
        self.horizontalLayout_2.addWidget(self.bBackControl)
        self.bInfo = QtWidgets.QPushButton(Q7DiagWindow)
        self.bInfo.setMinimumSize(QtCore.QSize(25, 25))
        self.bInfo.setMaximumSize(QtCore.QSize(25, 25))
        self.bInfo.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/icons/help-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bInfo.setIcon(icon2)
        self.bInfo.setObjectName("bInfo")
        self.horizontalLayout_2.addWidget(self.bInfo)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.bClose = QtWidgets.QPushButton(Q7DiagWindow)
        self.bClose.setObjectName("bClose")
        self.horizontalLayout_2.addWidget(self.bClose)
        self.gridLayout.addLayout(self.horizontalLayout_2, 5, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.bExpandAll = QtWidgets.QPushButton(Q7DiagWindow)
        self.bExpandAll.setMinimumSize(QtCore.QSize(25, 25))
        self.bExpandAll.setMaximumSize(QtCore.QSize(25, 25))
        self.bExpandAll.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/icons/level-in.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bExpandAll.setIcon(icon3)
        self.bExpandAll.setObjectName("bExpandAll")
        self.horizontalLayout.addWidget(self.bExpandAll)
        self.bCollapseAll = QtWidgets.QPushButton(Q7DiagWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(25)
        sizePolicy.setVerticalStretch(25)
        sizePolicy.setHeightForWidth(self.bCollapseAll.sizePolicy().hasHeightForWidth())
        self.bCollapseAll.setSizePolicy(sizePolicy)
        self.bCollapseAll.setMinimumSize(QtCore.QSize(25, 25))
        self.bCollapseAll.setMaximumSize(QtCore.QSize(25, 25))
        self.bCollapseAll.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/icons/level-out.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bCollapseAll.setIcon(icon4)
        self.bCollapseAll.setObjectName("bCollapseAll")
        self.horizontalLayout.addWidget(self.bCollapseAll)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.bPrevious = QtWidgets.QPushButton(Q7DiagWindow)
        self.bPrevious.setMinimumSize(QtCore.QSize(25, 25))
        self.bPrevious.setMaximumSize(QtCore.QSize(25, 25))
        self.bPrevious.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/icons/node-sids-opened.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bPrevious.setIcon(icon5)
        self.bPrevious.setObjectName("bPrevious")
        self.horizontalLayout.addWidget(self.bPrevious)
        self.eCount = QtWidgets.QLineEdit(Q7DiagWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.eCount.sizePolicy().hasHeightForWidth())
        self.eCount.setSizePolicy(sizePolicy)
        self.eCount.setMaximumSize(QtCore.QSize(30, 16777215))
        self.eCount.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.eCount.setReadOnly(True)
        self.eCount.setObjectName("eCount")
        self.horizontalLayout.addWidget(self.eCount)
        self.bNext = QtWidgets.QPushButton(Q7DiagWindow)
        self.bNext.setMinimumSize(QtCore.QSize(25, 25))
        self.bNext.setMaximumSize(QtCore.QSize(25, 25))
        self.bNext.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/images/icons/selected.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bNext.setIcon(icon6)
        self.bNext.setObjectName("bNext")
        self.horizontalLayout.addWidget(self.bNext)
        self.cFilter = QtWidgets.QComboBox(Q7DiagWindow)
        self.cFilter.setObjectName("cFilter")
        self.horizontalLayout.addWidget(self.cFilter)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.bWhich = QtWidgets.QPushButton(Q7DiagWindow)
        self.bWhich.setMinimumSize(QtCore.QSize(25, 25))
        self.bWhich.setMaximumSize(QtCore.QSize(25, 25))
        self.bWhich.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/images/icons/check-grammars.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bWhich.setIcon(icon7)
        self.bWhich.setIconSize(QtCore.QSize(24, 24))
        self.bWhich.setObjectName("bWhich")
        self.horizontalLayout.addWidget(self.bWhich)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.cWarnings = QtWidgets.QCheckBox(Q7DiagWindow)
        self.cWarnings.setEnabled(True)
        self.cWarnings.setChecked(True)
        self.cWarnings.setObjectName("cWarnings")
        self.horizontalLayout.addWidget(self.cWarnings)
        self.cDiagFirst = QtWidgets.QCheckBox(Q7DiagWindow)
        self.cDiagFirst.setEnabled(True)
        self.cDiagFirst.setObjectName("cDiagFirst")
        self.horizontalLayout.addWidget(self.cDiagFirst)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.bSave = QtWidgets.QPushButton(Q7DiagWindow)
        self.bSave.setMinimumSize(QtCore.QSize(25, 25))
        self.bSave.setMaximumSize(QtCore.QSize(25, 25))
        self.bSave.setText("")
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/images/icons/check-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSave.setIcon(icon8)
        self.bSave.setObjectName("bSave")
        self.horizontalLayout.addWidget(self.bSave)
        self.gridLayout.addLayout(self.horizontalLayout, 3, 0, 1, 1)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.diagTable = QtWidgets.QTreeWidget(Q7DiagWindow)
        self.diagTable.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.diagTable.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.diagTable.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.diagTable.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.diagTable.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.diagTable.setObjectName("diagTable")
        self.diagTable.headerItem().setText(0, "1")
        self.verticalLayout.addWidget(self.diagTable)
        self.gridLayout.addLayout(self.verticalLayout, 4, 0, 1, 1)

        self.retranslateUi(Q7DiagWindow)
        QtCore.QMetaObject.connectSlotsByName(Q7DiagWindow)

    def retranslateUi(self, Q7DiagWindow):
        _translate = QtCore.QCoreApplication.translate
        Q7DiagWindow.setWindowTitle(_translate("Q7DiagWindow", "Form"))
        self.bClose.setText(_translate("Q7DiagWindow", "Close"))
        self.cWarnings.setText(_translate("Q7DiagWindow", "Warnings"))
        self.cDiagFirst.setText(_translate("Q7DiagWindow", "Diagnostics first"))
from . import Res_rc



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  ---------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as C
import CGNS.PAT.cgnserrors as E
import CGNS.PAT.cgnskeywords as K
import numpy as N

#
data = C.newBCData(None, "{BCData}")
C.newDescriptor(data, "{Descriptor}")
C.newDataArray(data, "{DataLocal}")
C.newDataArray(data, "{DataGlobal}")
C.newDataClass(data)
C.newDimensionalUnits(data)
C.newUserDefinedData(data, "{UserDefinedData}")
#
status = "9.5"
comment = "Full SIDS with all optionals"
pattern = [data, status, comment]
#



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
import CGNS.VAL.simplecheck


def check(
    t, trace=False, grammar=None, paths=[""], stop=False, warnings=[], failures=[]
):
    if grammar is None:
        grammar = ["SIDS"]
    chk = CGNS.VAL.simplecheck.compliant(
        t,
        userlist=grammar,
        paths=paths,
        stop=stop,
        warnings=warnings,
        failures=failures,
    )
    if trace:
        for d in chk[1]:
            print("# %s >>> %s" % (d[1], d[0]))
    return chk[0]



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CGNS/NAV/T/Q7ControlWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Q7ControlWindow(object):
    def setupUi(self, Q7ControlWindow):
        Q7ControlWindow.setObjectName("Q7ControlWindow")
        Q7ControlWindow.resize(799, 232)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Q7ControlWindow.sizePolicy().hasHeightForWidth())
        Q7ControlWindow.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/icons/cgSpy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Q7ControlWindow.setWindowIcon(icon)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Q7ControlWindow)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.bTreeLoadLast = QtWidgets.QToolButton(Q7ControlWindow)
        self.bTreeLoadLast.setMinimumSize(QtCore.QSize(25, 25))
        self.bTreeLoadLast.setMaximumSize(QtCore.QSize(25, 25))
        self.bTreeLoadLast.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/icons/tree-load-g.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bTreeLoadLast.setIcon(icon1)
        self.bTreeLoadLast.setObjectName("bTreeLoadLast")
        self.horizontalLayout.addWidget(self.bTreeLoadLast)
        self.bTreeLoad = QtWidgets.QToolButton(Q7ControlWindow)
        self.bTreeLoad.setMinimumSize(QtCore.QSize(25, 25))
        self.bTreeLoad.setMaximumSize(QtCore.QSize(25, 25))
        self.bTreeLoad.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/icons/tree-load.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bTreeLoad.setIcon(icon2)
        self.bTreeLoad.setObjectName("bTreeLoad")
        self.horizontalLayout.addWidget(self.bTreeLoad)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.bEditTree = QtWidgets.QToolButton(Q7ControlWindow)
        self.bEditTree.setMinimumSize(QtCore.QSize(25, 25))
        self.bEditTree.setMaximumSize(QtCore.QSize(25, 25))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/icons/tree-new.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bEditTree.setIcon(icon3)
        self.bEditTree.setObjectName("bEditTree")
        self.horizontalLayout.addWidget(self.bEditTree)
        self.bPatternView = QtWidgets.QToolButton(Q7ControlWindow)
        self.bPatternView.setMinimumSize(QtCore.QSize(25, 25))
        self.bPatternView.setMaximumSize(QtCore.QSize(25, 25))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/icons/pattern.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bPatternView.setIcon(icon4)
        self.bPatternView.setObjectName("bPatternView")
        self.horizontalLayout.addWidget(self.bPatternView)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.bLog = QtWidgets.QPushButton(Q7ControlWindow)
        self.bLog.setMinimumSize(QtCore.QSize(25, 25))
        self.bLog.setMaximumSize(QtCore.QSize(25, 25))
        self.bLog.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/icons/subtree-sids-warning.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bLog.setIcon(icon5)
        self.bLog.setObjectName("bLog")
        self.horizontalLayout.addWidget(self.bLog)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.bOptionView = QtWidgets.QToolButton(Q7ControlWindow)
        self.bOptionView.setMinimumSize(QtCore.QSize(25, 25))
        self.bOptionView.setMaximumSize(QtCore.QSize(25, 25))
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/images/icons/options-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bOptionView.setIcon(icon6)
        self.bOptionView.setObjectName("bOptionView")
        self.horizontalLayout.addWidget(self.bOptionView)
        self.bInfo = QtWidgets.QPushButton(Q7ControlWindow)
        self.bInfo.setMinimumSize(QtCore.QSize(25, 25))
        self.bInfo.setMaximumSize(QtCore.QSize(25, 25))
        self.bInfo.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/images/icons/help-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bInfo.setIcon(icon7)
        self.bInfo.setObjectName("bInfo")
        self.horizontalLayout.addWidget(self.bInfo)
        self.bAbout = QtWidgets.QToolButton(Q7ControlWindow)
        self.bAbout.setMinimumSize(QtCore.QSize(25, 25))
        self.bAbout.setMaximumSize(QtCore.QSize(25, 25))
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/images/icons/view-help.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bAbout.setIcon(icon8)
        self.bAbout.setObjectName("bAbout")
        self.horizontalLayout.addWidget(self.bAbout)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.bClose = QtWidgets.QPushButton(Q7ControlWindow)
        self.bClose.setMinimumSize(QtCore.QSize(25, 25))
        self.bClose.setMaximumSize(QtCore.QSize(25, 25))
        self.bClose.setText("")
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/images/icons/close-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bClose.setIcon(icon9)
        self.bClose.setObjectName("bClose")
        self.horizontalLayout.addWidget(self.bClose)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.controlTable = Q7ControlTableWidget(Q7ControlWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.controlTable.sizePolicy().hasHeightForWidth())
        self.controlTable.setSizePolicy(sizePolicy)
        self.controlTable.setMinimumSize(QtCore.QSize(781, 181))
        self.controlTable.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.controlTable.setTabKeyNavigation(False)
        self.controlTable.setProperty("showDropIndicator", False)
        self.controlTable.setDragDropOverwriteMode(False)
        self.controlTable.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.controlTable.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.controlTable.setCornerButtonEnabled(False)
        self.controlTable.setRowCount(0)
        self.controlTable.setColumnCount(6)
        self.controlTable.setObjectName("controlTable")
        self.controlTable.horizontalHeader().setCascadingSectionResizes(True)
        self.controlTable.horizontalHeader().setStretchLastSection(True)
        self.verticalLayout_2.addWidget(self.controlTable)

        self.retranslateUi(Q7ControlWindow)
        QtCore.QMetaObject.connectSlotsByName(Q7ControlWindow)

    def retranslateUi(self, Q7ControlWindow):
        _translate = QtCore.QCoreApplication.translate
        Q7ControlWindow.setWindowTitle(_translate("Q7ControlWindow", "Form"))
        self.bTreeLoadLast.setToolTip(_translate("Q7ControlWindow", "Load the last used CGNS file"))
        self.bTreeLoad.setToolTip(_translate("Q7ControlWindow", "Load an existing CGNS file"))
        self.bEditTree.setToolTip(_translate("Q7ControlWindow", "Create a new CGNS tree from scratch"))
        self.bEditTree.setText(_translate("Q7ControlWindow", "..."))
        self.bPatternView.setToolTip(_translate("Q7ControlWindow", "Open CGNS/SIDS sub-trees database"))
        self.bPatternView.setText(_translate("Q7ControlWindow", "..."))
        self.bLog.setToolTip(_translate("Q7ControlWindow", "Log window"))
        self.bOptionView.setToolTip(_translate("Q7ControlWindow", "Set user defined options"))
        self.bOptionView.setText(_translate("Q7ControlWindow", "..."))
        self.bInfo.setToolTip(_translate("Q7ControlWindow", "Help window"))
        self.bAbout.setToolTip(_translate("Q7ControlWindow", "About...."))
        self.bAbout.setText(_translate("Q7ControlWindow", "..."))
        self.bClose.setToolTip(_translate("Q7ControlWindow", "Close all CGNS.NAV windows"))
        self.controlTable.setSortingEnabled(True)
from CGNS.NAV.mcontrol import Q7ControlTableWidget
from . import Res_rc



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  ---------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as C
import CGNS.PAT.cgnserrors as E
import CGNS.PAT.cgnskeywords as K
import numpy as N

data = C.newBase(None, "{Base}", 3, 3)
C.newZone(data, "{Zone}", N.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
C.newSimulationType(data)
C.newIntegralData(data, "{IntegralData}")
C.newBaseIterativeData(data, "{BaseIterativeData}")
C.newConvergenceHistory(data)
C.newFamily(data, "{Family}")
C.newFlowEquationSet(data)
C.newReferenceState(data)
C.newAxisymmetry(data)
C.newRotatingCoordinates(data)
C.newGravity(data)
C.newDataClass(data)
C.newDimensionalUnits(data)
C.newUserDefinedData(data, "{UserDefinedData}")
C.newDescriptor(data, "{Descriptor}")

status = "6.2"
comment = "Full SIDS with all optionals children"
pattern = [data, status, comment]



#!/usr/bin/env python
#  -------------------------------------------------------------------------
#  pyCGNS.APP - Python package for CFD General Notation System - APPlicater
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.MAP
from CGNS.PAT.cgnskeywords import *
import numpy
import os

filename = "/tmp/T0.cgns"

try:
    os.unlink(filename)
except os.error:
    pass

# this is a small example, because it's quite hard to read!
tree = [
    CGNSTree_s,
    None,
    [
        [
            CGNSLibraryVersion_s,
            numpy.array([2.4], dtype=numpy.float32),
            [],
            CGNSLibraryVersion_ts,
        ],
        [
            "Helicopter",
            numpy.array([3, 3], dtype=numpy.int32),
            [
                [
                    "ReferenceState",
                    None,
                    [
                        ["AngleofAttack", numpy.array([7.0]), [], DataArray_ts],
                        ["BetaAngle", numpy.array([0.0]), [], DataArray_ts],
                        ["Coef_Area", numpy.array([0.025]), [], DataArray_ts],
                    ],
                    ReferenceState_ts,
                ],
            ],
            CGNSBase_ts,
        ],
    ],
    CGNSTree_ts,
]

CGNS.MAP.save(filename, tree, links=[], flags=CGNS.MAP.S2P_DEFAULT)

# --- last line



#!/usr/bin/env python
#  -------------------------------------------------------------------------
#  pyCGNS.APP - Python package for CFD General Notation System - APPlicater
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.MAP

(tree, links, paths) = CGNS.MAP.load("./data/T0.cgns", flags=CGNS.MAP.S2P_DEFAULT)

print(tree)
print(links)

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = "grid 1D"
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, "{Base}", 1, 1)
z = CGL.newZone(b, "{Zone}", NPY.array([[5, 4, 0]], order="F"))
g = CGL.newGridCoordinates(z, "GridCoordinates")
d = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5,), dtype="float64", order="F"))
g = CGL.newGridCoordinates(z, "{Grid#002}")
d = CGL.newDataArray(g, CGK.CoordinateR_s, NPY.ones((5,), dtype="float64", order="F"))
g = CGL.newGridCoordinates(z, "{Grid#003}")
d = CGL.newDataArray(g, CGK.CoordinateXi_s, NPY.ones((5,), dtype="float64", order="F"))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "grid 2D"
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, "{Base}", 2, 2)
z = CGL.newZone(b, "{Zone}", NPY.array([[5, 4, 0], [7, 6, 0]], order="F"))
g = CGL.newGridCoordinates(z, "GridCoordinates")
d = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5, 7), dtype="float64", order="F"))
d = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((5, 7), dtype="float64", order="F"))
g = CGL.newGridCoordinates(z, "{Grid#002}")
d = CGL.newDataArray(
    g, CGK.CoordinateXi_s, NPY.ones((5, 7), dtype="float64", order="F")
)
d = CGL.newDataArray(
    g, CGK.CoordinateEta_s, NPY.ones((5, 7), dtype="float64", order="F")
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "grid 3D #1"
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, "{Base}", 3, 3)
z = CGL.newZone(b, "{Zone}", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
g = CGL.newGridCoordinates(z, "GridCoordinates")
d = CGL.newDataArray(
    g, CGK.CoordinateX_s, NPY.ones((5, 7, 9), dtype="float64", order="F")
)
d = CGL.newDataArray(
    g, CGK.CoordinateY_s, NPY.ones((5, 7, 9), dtype="float64", order="F")
)
d = CGL.newDataArray(
    g, CGK.CoordinateZ_s, NPY.ones((5, 7, 9), dtype="float64", order="F")
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "grid 3D #2"
diag = True
T = CGL.newCGNSTree()
b = CGL.newBase(T, "{Base}", 2, 3)
z = CGL.newZone(b, "{Zone}", NPY.array([[5, 4, 0], [7, 6, 0]], order="F"))
g = CGL.newGridCoordinates(z, "GridCoordinates")
d = CGL.newDataArray(g, CGK.CoordinateX_s, NPY.ones((5, 7), dtype="float64", order="F"))
d = CGL.newDataArray(g, CGK.CoordinateY_s, NPY.ones((5, 7), dtype="float64", order="F"))
d = CGL.newDataArray(g, CGK.CoordinateZ_s, NPY.ones((5, 7), dtype="float64", order="F"))
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "grid empty #1"
diag = False
T = CGL.newCGNSTree()
b = CGL.newBase(T, "{Base}", 3, 3)
z = CGL.newZone(b, "{Zone}", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
g = CGL.newGridCoordinates(z, "GridCoordinates")
g = CGL.newGridCoordinates(z, "{Grid#002}")
g = CGL.newGridCoordinates(z, "{Grid#003}")
g = CGL.newGridCoordinates(z, "{Grid#004}")
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "grid empty #2"
diag = False
T = CGL.newCGNSTree()
b = CGL.newBase(T, "{Base}", 3, 3)
z = CGL.newZone(b, "{Zone}", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
g = CGL.newGridCoordinates(z, "GridCoordinates")
r = CGL.newUserDefinedData(g, "{UserDefinedData}")
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "grid bad dims #1"
diag = False
T = CGL.newCGNSTree()
b = CGL.newBase(T, "{Base}", 3, 3)
z = CGL.newZone(b, "{Zone}", NPY.array([[5, 4, 0], [7, 6, 0], [9, 8, 0]], order="F"))
g = CGL.newGridCoordinates(z, "GridCoordinates")
d = CGL.newDataArray(
    g, CGK.CoordinateX_s, NPY.ones((4, 7, 9), dtype="float64", order="F")
)
d = CGL.newDataArray(
    g, CGK.CoordinateY_s, NPY.ones((5, 7, 9), dtype="float64", order="F")
)
d = CGL.newDataArray(
    g, CGK.CoordinateZ_s, NPY.ones((5, 7, 9), dtype="float64", order="F")
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#

from qtpy.QtCore import Qt
from qtpy.QtWidgets import QTableWidget

COPYPATTERN = '@@COPYPATTERN@@'

KEYMAPPING = {
    COPYPATTERN: Qt.Key_Space,
}


# -----------------------------------------------------------------
class Q7PatternTableWidget(QTableWidget):
    def mousePressEvent(self, event):
        self.lastPos = event.globalPos()
        self.lastButton = event.button()
        QTableWidget.mousePressEvent(self, event)

    def keyPressEvent(self, event):
        kmod = event.modifiers()
        kval = event.key()
        if kval == KEYMAPPING[COPYPATTERN]:
            self._panel.copyPatternInBuffer()
            return
        QTableWidget.keyPressEvent(self, event)

# -----------------------------------------------------------------



#!/usr/bin/env python
#  -------------------------------------------------------------------------
#  pyCGNS.APP - Python package for CFD General Notation System - APPlicater
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.MAP
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK

import time

# - load a single node in a tree
#   Nothing else but the target node and its ancestors are load
#   You load what you need, see below to get tree structure without data

path = "/Disk/zone1/GridCoordinates/CoordinateZ"
(tree, lk) = CGNS.MAP.load("data/T0.cgns", CGNS.MAP.S2P_DEFAULT, 0, path, [], None)
print(tree)
print("-" * 50)

updict = {}
path = "/Disk/zone1/GridCoordinates/CoordinateZ"
updict[path] = CGU.getValueByPath(tree, path)
path = "/Disk/zone2/GridCoordinates/CoordinateX"
updict[path] = CGU.getValueByPath(tree, path)

(tree, lk) = CGNS.MAP.load("data/T0.cgns", CGNS.MAP.S2P_DEFAULT, 0, None, [], updict)
print(tree)
print("-" * 50)

# - load only up to argument depth
#   Useful in case you only want the layout of the tree, if you want to
#   make links to already existing trees
#   If you set depth to 0 there is no depth limit
#   A level of 1 means only CGNSTree_t
#   A level of 2 means CGNSLibraryVersion_t, CGNSBase_t...
#   and so on
depth = 3
(tree, lk) = CGNS.MAP.load("data/T0.cgns", CGNS.MAP.S2P_DEFAULT, depth, None, [], None)
print(tree)
print("-" * 50)

# - do not load data but only tree skeletton
#   All data would be MT (no data, no data type)
flags = CGNS.MAP.S2P_DEFAULT | CGNS.MAP.S2P_NODATA
(tree, lk) = CGNS.MAP.load("data/T0.cgns", flags, 0, None, [], None)
print
tree

# find all zones
zlist = CGU.getAllNodesByTypeList2([CGK.Zone_ts], tree)

# load zone data
for z in zlist:
    print("-" * 50, z)
    (tree, lk) = CGNS.MAP.load("data/T0.cgns", CGNS.MAP.S2P_DEFAULT, 0, z, [], None)
    print(tree)

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS.VAL - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as CGL
import CGNS.PAT.cgnsutils as CGU
import CGNS.PAT.cgnskeywords as CGK
import numpy as NPY

TESTS = []

#  -------------------------------------------------------------------------
tag = "elements"
diag = True


def makeCorrectTree(vertexsize, cellsize):
    T = CGL.newCGNSTree()
    b = CGL.newBase(T, "Base", 3, 3)
    s = NPY.array([[vertexsize, cellsize, 0]], dtype="int32", order="F")
    z = CGL.newZone(b, "Zone", s, CGK.Unstructured_s)
    g = CGL.newGridCoordinates(z, "GridCoordinates")
    d = CGL.newDataArray(
        g, CGK.CoordinateX_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateY_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    d = CGL.newDataArray(
        g, CGK.CoordinateZ_s, NPY.ones((vertexsize), dtype="float64", order="F")
    )
    return (T, b, z)


vertexsize = 20
cellsize = 7
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements bad elementsizeboundary"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
tetras[1][1] = cellsize
zbc = CGL.newZoneBC(z)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements elementsizeboundary BC correctly defined"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
tetras[1][1] = cellsize - 1
zbc = CGL.newZoneBC(z)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(1, cellsize)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements elementsizeboundary faces not on BC/GC (Warning)"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
tetras[1][1] = cellsize - 1
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements elementsizeboundary bndfaces multiply defined on BC (Warning)"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
tetras[1][1] = cellsize - 1
zbc = CGL.newZoneBC(z)
n = CGL.newBoundary(
    zbc,
    "BC1",
    [range(1, cellsize)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
n = CGL.newBoundary(
    zbc,
    "BC2",
    [range(2, cellsize - 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.CellCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements elementsizeboundary bndfaces multiply defined on BC/GC (Warning)"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
ntris = 12
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
tris = CGL.newElements(
    z,
    "TRIS",
    CGK.TRI_3_s,
    NPY.array([[cellsize + 1, cellsize + ntris]], "i", order="F"),
    NPY.ones((ntris * 3), dtype="int32"),
)
tris[1][1] = ntris
zbc = CGL.newZoneBC(z)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(cellsize + 1, cellsize + ntris + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
z2 = CGU.copyNode(z, "Zone2")
b[2].append(z2)
zgc = CGL.newZoneGridConnectivity(z)
gc = CGL.newGridConnectivity(zgc, "join1_2", "Zone2", ctype=CGK.Abutting1to1_s)
CGL.newIndexArray(
    gc,
    CGK.PointList_s,
    value=NPY.array([range(cellsize + 2, cellsize + ntris)], order="F"),
)
CGL.newIndexArray(
    gc,
    CGK.PointListDonor_s,
    value=NPY.array([range(cellsize + 2, cellsize + ntris)], order="F"),
)
CGL.newGridLocation(gc, value=CGK.FaceCenter_s)
zgc = CGL.newZoneGridConnectivity(z2)
gc = CGL.newGridConnectivity(zgc, "join2_1", "Zone", ctype=CGK.Abutting1to1_s)
CGL.newIndexArray(
    gc,
    CGK.PointList_s,
    value=NPY.array([range(cellsize + 2, cellsize + ntris)], order="F"),
)
CGL.newIndexArray(
    gc,
    CGK.PointListDonor_s,
    value=NPY.array([range(cellsize + 2, cellsize + ntris)], order="F"),
)
CGL.newGridLocation(gc, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements absent children"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
CGU.removeChildByName(tetras, CGK.ElementRange_s)  # ElementRange child absent
CGU.removeChildByName(
    tetras, CGK.ElementConnectivity_s
)  # ElementConnectivity child absent
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements out of range"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
tetras[2][0][1][0] = vertexsize + 1  # ElementConnectity element out of range
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements bad child shape"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones(((cellsize - 1) * 4), dtype="int32"),
)  # bad ElementConnectivity node shape
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements correct NGON shape"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
element = CGL.newElements(
    z,
    "NGON",
    CGK.NGON_n_s,
    NPY.array([[cellsize + 1, cellsize + cellsize]], "i", order="F"),
    NPY.array(
        [
            4,
            9,
            9,
            9,
            9,
            5,
            9,
            9,
            9,
            9,
            9,
            3,
            9,
            9,
            9,
            5,
            9,
            9,
            9,
            9,
            9,
            3,
            9,
            9,
            9,
            5,
            9,
            9,
            9,
            9,
            9,
            4,
            9,
            9,
            9,
            9,
        ],
        dtype="int32",
        order="F",
    ),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements uncorrect NGON shape"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 4), dtype="int32"),
)
element = CGL.newElements(
    z,
    "NGON",
    CGK.NGON_n_s,
    NPY.array([[cellsize + 1, cellsize + cellsize]], "i", order="F"),
    NPY.array(
        [4, 9, 9, 9, 9, 5, 9, 9, 9, 9, 9, 3, 9, 9, 9, 5, 9, 9, 9, 9, 9, 4, 9, 9, 9, 9],
        dtype="int32",
        order="F",
    ),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements correct MIXED shape"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
element = CGL.newElements(
    z,
    "MIXED",
    CGK.MIXED_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.array(
        [
            CGK.ElementType[CGK.QUAD_4_s],
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.PYRA_5_s],
            9,
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.TRI_3_s],
            9,
            9,
            9,
            CGK.ElementType[CGK.PYRA_5_s],
            9,
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.TRI_3_s],
            9,
            9,
            9,
            CGK.ElementType[CGK.PYRA_5_s],
            9,
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.QUAD_4_s],
            9,
            9,
            9,
            9,
        ],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 5, 11, 15, 21, 25, 31, 36], dtype="int32", order="F"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements uncorrect MIXED shape #1"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
element = CGL.newElements(
    z,
    "MIXED",
    CGK.MIXED_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.array(
        [
            CGK.ElementType[CGK.QUAD_4_s],
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.PYRA_5_s],
            9,
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.TRI_3_s],
            9,
            9,
            9,
            CGK.ElementType[CGK.PYRA_5_s],
            9,
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.QUAD_4_s],
            9,
            9,
            9,
            9,
        ],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 5, 11, 15, 21, 26], dtype="int32", order="F"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements uncorrect MIXED shape #2"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
element = CGL.newElements(
    z,
    "MIXED",
    CGK.MIXED_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.array(
        [
            CGK.ElementType[CGK.QUAD_4_s],
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.PYRA_5_s],
            9,
            9,
            9,
            9,
            9,
            999,
            CGK.ElementType[CGK.TRI_3_s],
            9,
            9,
            9,
            CGK.ElementType[CGK.PYRA_5_s],
            9,
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.QUAD_4_s],
            9,
            9,
            9,
            9,
        ],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 5, 12, 16, 22, 27], dtype="int32", order="F"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements uncorrect element within MIXED"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
element = CGL.newElements(
    z,
    "MIXED",
    CGK.MIXED_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.array(
        [
            CGK.ElementType[CGK.QUAD_4_s],
            9,
            9,
            9,
            9,
            999,
            9,
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.TRI_3_s],
            9,
            9,
            9,
            CGK.ElementType[CGK.PYRA_5_s],
            9,
            9,
            9,
            9,
            9,
            CGK.ElementType[CGK.QUAD_4_s],
            9,
            9,
            9,
            9,
        ],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 11, 15, 21, 26], dtype="int32", order="F"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements FACE"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nface = CGL.newElements(
    z,
    "NFACE",
    CGK.NFACE_n_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.array(
        [
            cellsize + 1,
            cellsize + 2,
            -cellsize - 3,
            cellsize + 4,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            cellsize + 5,
            cellsize + 1,
            -cellsize - 2,
            cellsize + 3,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            -cellsize - 5,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            cellsize + 5,
            cellsize + 1,
            cellsize + 2,
            -cellsize - 3,
            cellsize + 4,
        ],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 4, 9, 12, 17, 20, 25, 29], dtype="int32", order="F"),
)
ngon = CGL.newElements(
    z,
    "NGON",
    CGK.NGON_n_s,
    NPY.array([[cellsize + 1, cellsize + 5]], "i", order="F"),
    NPY.array(
        [9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 4, 9, 12, 17, 21], dtype="int32", order="F"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements FACE inconsistent dataarray"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nface = CGL.newElements(
    z,
    "NFACE",
    CGK.NFACE_n_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.array(
        [
            cellsize + 1,
            cellsize + 2,
            -cellsize - 3,
            cellsize + 4,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            cellsize + 5,
            cellsize + 1,
            -cellsize - 2,
            cellsize + 3,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            -cellsize - 5,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            cellsize + 5,
            cellsize + 1,
            cellsize + 2,
            -cellsize - 3,
            cellsize + 4,
        ],
        dtype="int32",
        order="F",
    ),
    NPY.array(
        [0, 4, 9999997, 10000000, 10000005, 10000008, 10000013, 10000017],
        dtype="int32",
        order="F",
    ),
)
ngon = CGL.newElements(
    z,
    "NGON",
    CGK.NGON_n_s,
    NPY.array([[cellsize + 1, cellsize + 5]], "i", order="F"),
    NPY.array(
        [9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 4, 9, 12, 17, 21], dtype="int32", order="F"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements FACE bad node shape"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nface = CGL.newElements(
    z,
    "NFACE",
    CGK.NFACE_n_s,
    NPY.array([[1, cellsize - 1]], "i", order="F"),
    NPY.array(
        [
            cellsize + 1,
            cellsize + 2,
            -cellsize - 3,
            cellsize + 4,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            cellsize + 5,
            cellsize + 1,
            -cellsize - 2,
            cellsize + 3,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            -cellsize - 5,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            cellsize + 5,
            cellsize + 1,
            cellsize + 2,
            -cellsize - 3,
            cellsize + 4,
        ],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 4, 9, 12, 17, 20, 25, 29], dtype="int32", order="F"),
)
ngon = CGL.newElements(
    z,
    "NGON",
    CGK.NGON_n_s,
    NPY.array([[cellsize + 1, cellsize + 5]], "i", order="F"),
    NPY.array(
        [9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 4, 9, 12, 17, 21], dtype="int32", order="F"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements FACE face index out of range"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nface = CGL.newElements(
    z,
    "NFACE",
    CGK.NFACE_n_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.array(
        [
            cellsize + 1,
            cellsize + 2,
            -cellsize - 3,
            cellsize + 4,
            cellsize + 1,
            cellsize + 2,
            999999,
            cellsize + 4,
            cellsize + 5,
            cellsize + 1,
            -cellsize - 2,
            cellsize + 3,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            -cellsize - 5,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 1,
            cellsize + 2,
            cellsize + 3,
            cellsize + 4,
            cellsize + 5,
            cellsize + 1,
            cellsize + 2,
            -cellsize - 3,
            cellsize + 4,
        ],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 4, 9, 12, 17, 20, 25, 29], dtype="int32", order="F"),
)
ngon = CGL.newElements(
    z,
    "NGON",
    CGK.NGON_n_s,
    NPY.array([[cellsize + 1, cellsize + 5]], "i", order="F"),
    NPY.array(
        [9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9],
        dtype="int32",
        order="F",
    ),
    NPY.array([0, 4, 9, 12, 17, 21], dtype="int32", order="F"),
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelements and parentelementsposition"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 8), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
pe = CGL.newParentElements(quads, NPY.ones((nquads, 2), dtype="int32", order="F"))
pp = CGL.newParentElementsPosition(
    quads, NPY.ones((nquads, 2), dtype="int32", order="F")
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelementsposition without parentelements"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 8), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
pp = CGL.newParentElementsPosition(
    quads, NPY.ones((nquads, 2), dtype="int32", order="F")
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements bad parentelements shape and parentelementsposition datatype"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 8), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
pe = CGL.newParentElements(quads, NPY.ones((nquads - 1, 2), dtype="int32", order="F"))
pp = CGL.newParentElementsPosition(
    quads, NPY.ones((nquads, 2), dtype="float64", order="F")
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelements and parentelementsposition on bad element type (Warning)"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 8), dtype="int32"),
)
pe = CGL.newParentElements(tetras, NPY.ones((cellsize, 2), dtype="int32", order="F"))
pp = CGL.newParentElementsPosition(
    tetras, NPY.ones((cellsize, 2), dtype="int32", order="F")
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelements bad values #1"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 8), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
pe = CGL.newParentElements(
    quads, NPY.array([[1, 2], [1, cellsize + 1], [1, 2]], dtype="int32", order="F")
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelements bad values #2"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nhexa = 2
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, nhexa]], "i", order="F"),
    NPY.ones((nhexa * 8), dtype="int32"),
)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[nhexa + 1, cellsize]], "i", order="F"),
    NPY.ones(((cellsize - nhexa) * 4), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
pe = CGL.newParentElements(quads, NPY.ones((nquads, 2), dtype="int32", order="F"))
pe[1][0][1] = nhexa + 1
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelements with boundary faces"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nhexa = 2
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, nhexa]], "i", order="F"),
    NPY.ones((nhexa * 8), dtype="int32"),
)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[nhexa + 1, cellsize]], "i", order="F"),
    NPY.ones(((cellsize - nhexa) * 4), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
pe = CGL.newParentElements(
    quads, NPY.array([[1, 0], [2, 0], [2, 0]], dtype="int32", order="F")
)
zbc = CGL.newZoneBC(z)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(cellsize + 1, cellsize + nquads + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelements with boundary faces but no BC or GC (warning)"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nhexa = 2
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, nhexa]], "i", order="F"),
    NPY.ones((nhexa * 8), dtype="int32"),
)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[nhexa + 1, cellsize]], "i", order="F"),
    NPY.ones(((cellsize - nhexa) * 4), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
pe = CGL.newParentElements(
    quads, NPY.array([[1, 0], [2, 0], [2, 0]], dtype="int32", order="F")
)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelements with boundary faces but badly defined"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nhexa = 2
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, nhexa]], "i", order="F"),
    NPY.ones((nhexa * 8), dtype="int32"),
)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[nhexa + 1, cellsize]], "i", order="F"),
    NPY.ones(((cellsize - nhexa) * 4), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
pe = CGL.newParentElements(quads, NPY.zeros((nquads, 2), dtype="int32", order="F"))
zbc = CGL.newZoneBC(z)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(cellsize + 1, cellsize + nquads + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelements and elementsizeboundary compatible"
diag = True
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nhexa = 2
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, nhexa]], "i", order="F"),
    NPY.ones((nhexa * 8), dtype="int32"),
)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[nhexa + 1, cellsize]], "i", order="F"),
    NPY.ones(((cellsize - nhexa) * 4), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
quads[1][1] = nquads
pe = CGL.newParentElements(
    quads, NPY.array([[1, 0], [2, 0], [2, 0]], dtype="int32", order="F")
)
zbc = CGL.newZoneBC(z)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(cellsize + 1, cellsize + nquads + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelements and elementsizeboundary not compatible"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
nhexa = 2
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, nhexa]], "i", order="F"),
    NPY.ones((nhexa * 8), dtype="int32"),
)
tetras = CGL.newElements(
    z,
    "TETRAS",
    CGK.TETRA_4_s,
    NPY.array([[nhexa + 1, cellsize]], "i", order="F"),
    NPY.ones(((cellsize - nhexa) * 4), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
quads[1][1] = nquads
pe = CGL.newParentElements(quads, NPY.ones((nquads, 2), dtype="int32", order="F"))
zbc = CGL.newZoneBC(z)
n = CGL.newBoundary(
    zbc,
    "BC",
    [range(cellsize + 1, cellsize + nquads + 1)],
    btype=CGK.Null_s,
    family=None,
    pttype=CGK.PointList_s,
)
g = CGL.newGridLocation(n, value=CGK.FaceCenter_s)
TESTS.append((tag, T, diag))

#  -------------------------------------------------------------------------
tag = "elements parentelementsposition bad face position"
diag = False
(T, b, z) = makeCorrectTree(vertexsize, cellsize)
hexas = CGL.newElements(
    z,
    "HEXAS",
    CGK.HEXA_8_s,
    NPY.array([[1, cellsize]], "i", order="F"),
    NPY.ones((cellsize * 8), dtype="int32"),
)
nquads = 3
quads = CGL.newElements(
    z,
    "QUADS",
    CGK.QUAD_4_s,
    NPY.array([[cellsize + 1, cellsize + nquads]], "i", order="F"),
    NPY.ones((nquads * 4), dtype="int32"),
)
pe = CGL.newParentElements(quads, NPY.ones((nquads, 2), dtype="int32", order="F"))
pp = CGL.newParentElementsPosition(
    quads, NPY.ones((nquads, 2), dtype="int32", order="F") * 7
)  # 7 > 6 faces for hexas
TESTS.append((tag, T, diag))



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#

import os

from qtpy.QtCore import Qt
from qtpy.QtGui import QFont
from qtpy.QtWidgets import QTreeView, QFileDialog, QTreeWidgetItem, QWidget


import CGNS.NAV.wmessages as MSG
import CGNS.VAL.parse.messages as CGM
from CGNS.NAV.Q7DiagWindow import Ui_Q7DiagWindow
from CGNS.NAV.moption import Q7OptionContext as OCTXT
from CGNS.NAV.wfingerprint import Q7Window as QW
from CGNS.VAL.parse.findgrammar import locateGrammars


# -----------------------------------------------------------------
class Q7CheckList(QW, Ui_Q7DiagWindow):
    def __init__(self, parent, data, fgindex):
        QW.__init__(self, QW.VIEW_DIAG, parent._control, None, fgindex)
        self._currentItem = 0
        self._filterItems = {}
        self.bClose.clicked.connect(self.reject)
        self.bExpandAll.clicked.connect(self.expand)
        self.bInfo.clicked.connect(self.infoDiagView)
        self.bCollapseAll.clicked.connect(self.collapse)
        self.bPrevious.clicked.connect(self.previousfiltered)
        self.bNext.clicked.connect(self.nextfiltered)
        self.cWarnings.clicked.connect(self.reset)
        self.cDiagFirst.clicked.connect(self.reset)
        self.bSave.clicked.connect(self.diagnosticssave)
        self.bWhich.clicked.connect(self.grammars)
        self.cFilter.currentIndexChanged[int].connect(self.filterChange)
        # QObject.connect(self.cFilter,
        #                 SIGNAL("currentIndexChanged(int)"),
        #                 self.filterChange)
        self._data = data
        self._parent = parent
        self.diagTable.keyPressEvent = self.diagTableKeyPressEvent
        self.filterChange()

    def doRelease(self):
        pass

    def grammars(self):
        self.FG.pushGrammarPaths()
        gl = ""
        for g in locateGrammars():
            gl += "%s : %s<br>" % (g[0], os.path.split(g[1])[0])
        self.FG.popGrammarPaths()
        MSG.wInfo(
            self,
            400,
            "Diag view:",
            """Checks performed using the following grammars:<br>%s""" % gl,
            again=False,
        )

    def diagTableKeyPressEvent(self, event):
        kmod = event.modifiers()
        kval = event.key()
        if kval == Qt.Key_Space:
            itlist = self.diagTable.selectedItems()
            it = itlist[0]
            itxt = it.text(0)
            if itxt[0] != "/":
                itxt = it.parent().text(0)
            self._parent.treeview.selectByPath(itxt)
        else:
            QTreeView.keyPressEvent(self.diagTable, event)

    def infoDiagView(self):
        self._control.helpWindow("Diagnosis")

    def diagnosticssave(self):
        n = "data=%s\n" % self._data

        filename = QFileDialog.getSaveFileName(self, "Save diagnosis", ".", "*.py")
        if filename[0] == "":
            return
        with open(str(filename[0]), "w+") as f:
            f.write(n)

    def previousfiltered(self):
        iold = self._filterItems[self.cFilter.currentText()][self._currentItem]
        if self._currentItem != 0:
            self._currentItem -= 1
        inew = self._filterItems[self.cFilter.currentText()][self._currentItem]
        iold.setSelected(False)
        inew.setSelected(True)
        self.diagTable.scrollToItem(inew)
        self.eCount.setText(str(self._currentItem + 1))

    def nextfiltered(self):
        ilist = self._filterItems[self.cFilter.currentText()]
        if self._currentItem >= len(ilist) - 1:
            return
        iold = ilist[self._currentItem]
        self._currentItem += 1
        inew = ilist[self._currentItem]
        iold.setSelected(False)
        inew.setSelected(True)
        self.diagTable.scrollToItem(inew)
        self.eCount.setText(str(self._currentItem + 1))

    def filterChange(self):
        self._currentItem = 0
        self.eCount.setText("")
        self.diagTable.clearSelection()

    def expand(self):
        self.diagTable.expandAll()

    def collapse(self):
        self.diagTable.collapseAll()

    def show(self):
        self.reset()
        super(Q7CheckList, self).show()

    def reset(self):
        v = self.diagTable
        v.clear()
        v.setHeaderHidden(True)
        plist = list(self._data)
        plist.sort()
        plist.reverse()
        keyset = set()
        self.cFilter.clear()
        self._filterItems = {}
        diagfirst = self.cDiagFirst.isChecked()
        warnings = self.cWarnings.isChecked()
        diagstack = {}
        for path in plist:
            path_item = None
            state = self._data.getWorstDiag(path)
            if state in [CGM.CHECK_NONE, CGM.CHECK_PASS]:
                pass
            elif (state == CGM.CHECK_WARN) and not warnings:
                pass
            else:
                for (diag, pth) in self._data.diagnosticsByPath(path):
                    if (diag.level == CGM.CHECK_WARN) and not warnings:
                        pass
                    elif diagfirst:
                        if diag.key not in keyset:
                            keyset.add(diag.key)
                            diag_item = self.addDiagEntry(None, diag, top=True)
                            v.insertTopLevelItem(0, diag_item)
                            diagstack[diag.key] = diag_item
                        path_item = self.addPathEntry(
                            diagstack[diag.key], path, state, top=False
                        )
                    else:
                        if diag.key not in keyset:
                            keyset.add(diag.key)
                        if path_item is None:
                            path_item = self.addPathEntry(None, path, state, top=True)
                            v.insertTopLevelItem(0, path_item)
                        diag_item = self.addDiagEntry(path_item, diag, top=False)
                    keylist = list(keyset)
        keylist.sort()
        for k in keylist:
            self.cFilter.addItem(k)

    def addPathEntry(self, parent, path, state, top=False):
        it = QTreeWidgetItem(parent, (path,))
        ft = QFont(OCTXT._Table_Font)
        if top:
            ft.setBold(True)
        it.setFont(0, ft)
        if state == CGM.CHECK_FAIL:
            it.setIcon(0, self.IC(QW.I_C_SFL))
        if state == CGM.CHECK_WARN:
            it.setIcon(0, self.IC(QW.I_C_SWR))
        return it

    def addDiagEntry(self, parent, diag, top=False):
        dit = QTreeWidgetItem(parent, (self._data.message(diag),))
        ft = QFont(OCTXT._Table_Font)
        if top:
            ft.setBold(True)
        dit.setFont(0, ft)
        if diag.level == CGM.CHECK_FAIL:
            dit.setIcon(0, self.IC(QW.I_C_SFL))
        if diag.level == CGM.CHECK_WARN:
            dit.setIcon(0, self.IC(QW.I_C_SWR))
        if diag.key not in self._filterItems:
            self._filterItems[diag.key] = [dit]
        else:
            self._filterItems[diag.key].insert(0, dit)
        return dit

    def reject(self):
        self.close()

    def close(self):
        self._parent.diagview = None
        QWidget.close(self)


# -----------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
from CGNS.NAV.moption import Q7OptionContext as OCTXT

import numpy

SCRIPT_PATTERN = """#!/usr/bin/env python
#  -------------------------------------------------------------------------
#  pyCGNS.NAV - Python package for CFD General Notation System - NAVigater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#  CGNS.NAV - GENERATED FILE - %(Q_VAR_DATE)s
#  query: %(Q_VAR_QUERYNAME)s
# -----------------------------------------------------------------
import CGNS.MAP as CGM

FILE='%(Q_VAR_TREE_FILE)s'
SCRIPT=\"\"\"%(Q_VAR_QUERY_SCRIPT)s\"\"\"
ARGS=\"\"\"%(Q_VAR_QUERY_ARGS)s\"\"\"
SELECTED=\"\"\"%(Q_VAR_SELECTED)s\"\"\"

# -----------------------------------------------------------------
SCRIPT_PRE=\"\"\"%(Q_VAR_SCRIPT_PRE)s\"\"\"
SCRIPT_POST=\"\"\"%(Q_VAR_SCRIPT_POST)s\"\"\"
# -----------------------------------------------------------------
def evalScript(node,parent,tree,links,skips,path,val,args,selected):
    l=locals()
    l['%(Q_VAR_RESULT_LIST)s']=[False]
    l['%(Q_VAR_PARENT)s']=parent
    l['%(Q_VAR_NAME)s']=node[0]
    l['%(Q_VAR_VALUE)s']=node[1]
    l['%(Q_VAR_CGNSTYPE)s']=node[3]
    l['%(Q_VAR_CHILDREN)s']=node[2]
    l['%(Q_VAR_TREE)s']=tree
    l['%(Q_VAR_LINKS)s']=links
    l['%(Q_VAR_SKIPS)s']=skips
    l['%(Q_VAR_SELECTED)s']=selected
    l['%(Q_VAR_PATH)s']=path
    if (args is None): args=()
    l['%(Q_VAR_USER)s']=args
    l['%(Q_VAR_NODE)s']=node
    pre=SCRIPT_PRE+val+SCRIPT_POST
    try:
      eval(compile(pre,'<string>','exec'),globals(),l)
    except Exception:
      l['%(Q_VAR_RESULT_LIST)s'][0]=False
    RESULT=l['%(Q_VAR_RESULT_LIST)s'][0]
    return RESULT
# -----------------------------------------------------------------
def parseAndSelect(tree,node,parent,links,skips,path,script,args,selected,
                   result):
    path=path+'/'+node[0]
    Q=evalScript(node,parent,tree,links,skips,path,script,args,selected)
    R=[]
    if (Q):
        if (result):
            R=[Q]
        else:
            R=[path]
    for C in node[2]:
        R+=parseAndSelect(tree,C,node,links,skips,path,script,args,selected,
                          result)
    return R

# -----------------------------------------------------------------
def run(tree,links,skips,mode,args,script,selected):
    v=None
    try:
        if (args): v=eval(args)
        if ((v is not None) and (type(v)!=tuple)): v=(v,)
    except NameError:
        v=(str(args),)
    except:
        pass
    _args=v
    result=parseAndSelect(tree,tree,[None,None,[],None],links,skips,'',
                          script,_args,selected,mode)
    return result
# -----------------------------------------------------------------
(t,l,p)=CGM.load(FILE)
print run(t,l,p,True,ARGS,SCRIPT,SELECTED)

# -----------------------------------------------------------------
"""


# -----------------------------------------------------------------
def sameVal(n, v):
    if n is None:
        if v is None:
            return True
        return False
    if n.dtype.char in ['S', 'c']:
        return n.tostring().decode('ascii') == v
    if n.dtype.char in ['d', 'f', 'i', 'l', 'q']:
        return n.flat[0] == v
    if n == v:
        return True
    return False


# -----------------------------------------------------------------
def sameValType(n, v):
    if isinstance(n, numpy.ndarray):
        return False
    if n.dtype.char == v:
        return True
    return False


# -----------------------------------------------------------------
def evalScript(node, parent, tree, links, skips, path, val, args, selected):
    l = locals()
    l[OCTXT.Q_VAR_RESULT_LIST] = [False]
    l[OCTXT.Q_VAR_PARENT] = parent
    l[OCTXT.Q_VAR_NAME] = node[0]
    l[OCTXT.Q_VAR_VALUE] = node[1]
    l[OCTXT.Q_VAR_CGNSTYPE] = node[3]
    l[OCTXT.Q_VAR_CHILDREN] = node[2]
    l[OCTXT.Q_VAR_TREE] = tree
    l[OCTXT.Q_VAR_LINKS] = links
    l[OCTXT.Q_VAR_SKIPS] = skips
    l[OCTXT.Q_VAR_PATH] = path
    l[OCTXT.Q_VAR_SELECTED] = selected
    if args is None:
        args = ()
    l[OCTXT.Q_VAR_USER] = args
    l[OCTXT.Q_VAR_NODE] = node
    pre = OCTXT.Q_SCRIPT_PRE + val + OCTXT.Q_SCRIPT_POST
    if OCTXT.QueryNoException:
        eval(compile(pre, '<string>', 'exec'), globals(), l)
    else:
        try:
            eval(compile(pre, '<string>', 'exec'), globals(), l)
        except Exception:
            l[OCTXT.Q_VAR_RESULT_LIST][0] = False
    RESULT = l[OCTXT.Q_VAR_RESULT_LIST][0]
    return RESULT


# -----------------------------------------------------------------
def parseAndSelect(tree, node, parent, links, skips, path, script, args, selected,
                   result):
    path = path + '/' + node[0]
    Q = evalScript(node, parent, tree, links, skips, path, script, args, selected)
    R = []
    if Q:
        if result:
            R = [Q]
        else:
            R = [path]
    for C in node[2]:
        R += parseAndSelect(tree, C, node, links, skips, path, script, args,
                            selected, result)
    return R


# -----------------------------------------------------------------
class Q7QueryEntry(object):
    def __init__(self, name, group=None, script='', doc='',
                 update=False, hasargs=False):
        self._name = name
        self._group = group
        self._script = script
        self._doc = doc
        self._update = update
        self._hasargs = hasargs

    @property
    def name(self):
        return self._name

    @property
    def group(self):
        return self._group

    @property
    def doc(self):
        return self._doc

    @property
    def hasArgs(self):
        return self._hasargs

    @property
    def script(self):
        return self._script

    def requireTreeUpdate(self):
        return self._update

    def setRequireTreeUpdate(self, value):
        self._update = value

    def setScript(self, value):
        self._script = value

    def setDoc(self, value):
        self._doc = value

    def __str__(self):
        s = '("%s","%s","%s","""%s""",%s)' % \
            (self.name, self.group, self._script, self._doc, self._update)
        return s

    def run(self, tree, links, skips, mode, args, selected=None):
        if selected is None:
            selected = []
        v = None
        try:
            if isinstance(args, numpy.ndarray):
                v = ('numpy.' + repr(args),)
            else:
                if args not in [None, [], ()]:
                    v = eval(args)
                if (v is not None) and not isinstance(v, tuple):
                    v = (v,)
        except NameError as e:
            v = (str(args),)
        except Exception as e:
            print(e)
        self._args = v
        result = parseAndSelect(tree, tree, [None, None, [], None], links, skips, '',
                                self._script, self._args, selected, mode)
        return result

    def getFullScript(self, filename, text, args):
        datadict = {}
        datadict['Q_VAR_DATE'] = '00/00/00'
        datadict['Q_VAR_QUERYNAME'] = '%s/%s' % (self._group, self._name)
        datadict['Q_VAR_SCRIPT_PRE'] = OCTXT.Q_SCRIPT_PRE
        datadict['Q_VAR_SCRIPT_POST'] = OCTXT.Q_SCRIPT_POST
        datadict['Q_VAR_RESULT_LIST'] = OCTXT.Q_VAR_RESULT_LIST
        datadict['Q_VAR_PARENT'] = OCTXT.Q_VAR_PARENT
        datadict['Q_VAR_NAME'] = OCTXT.Q_VAR_NAME
        datadict['Q_VAR_VALUE'] = OCTXT.Q_VAR_VALUE
        datadict['Q_VAR_CGNSTYPE'] = OCTXT.Q_VAR_CGNSTYPE
        datadict['Q_VAR_CHILDREN'] = OCTXT.Q_VAR_CHILDREN
        datadict['Q_VAR_TREE'] = OCTXT.Q_VAR_TREE
        datadict['Q_VAR_PATH'] = OCTXT.Q_VAR_PATH
        datadict['Q_VAR_LINKS'] = OCTXT.Q_VAR_LINKS
        datadict['Q_VAR_SKIPS'] = OCTXT.Q_VAR_SKIPS
        datadict['Q_VAR_USER'] = OCTXT.Q_VAR_USER
        datadict['Q_VAR_NODE'] = OCTXT.Q_VAR_NODE
        datadict['Q_VAR_SELECTED'] = OCTXT.Q_VAR_SELECTED
        datadict['Q_VAR_QUERY_SCRIPT'] = text
        datadict['Q_VAR_QUERY_ARGS'] = args
        datadict['Q_VAR_TREE_FILE'] = filename
        script = SCRIPT_PATTERN % datadict
        return script

# -----------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#

import sys

from qtpy.QtCore import (Qt, QAbstractTableModel)
from qtpy.QtGui import (QFont, QFontMetrics)
from qtpy.QtWidgets import QTableView

import CGNS.PAT.cgnskeywords as CGK
import CGNS.PAT.cgnsutils as CGU
from CGNS.NAV.moption import Q7OptionContext as OCTXT


# -----------------------------------------------------------------
class Q7TableView(QTableView):
    def __init__(self, parent):
        super(Q7TableView, self).__init__(None)
        self._parent = None
        self.lastPos = None
        self.lastButton = None

    def mousePressEvent(self, event):
        self.lastPos = event.globalPos()
        self.lastButton = event.button()
        QTableView.mousePressEvent(self, event)


# -----------------------------------------------------------------
class Q7TableModel(QAbstractTableModel):
    def __init__(self, node, showparams, parent=None):
        super(Q7TableModel, self).__init__(parent)
        self.node = node
        self.showmode = showparams['mode']
        self.cs = showparams['cs']
        self.ls = showparams['ls']
        self.fmt = "%-s"
        tx = self.fmt % 'w' * 16
        if self.node.sidsDataType() in [CGK.R4, CGK.R8]:
            self.fmt = "% 0.8e"
            tx = self.fmt % 9.9999999999999
        if self.node.sidsDataType() in [CGK.I4, CGK.I8]:
            self.fmt = "%8d"
            tx = self.fmt % 999999999
        if self.node.sidsDataType() in [CGK.C1]:
            self.fmt = "%1s"
            tx = self.fmt % 'w'
        if self.node.sidsDataType() not in [CGK.MT, CGK.LK]:
            if self.node.sidsValueFortranOrder():
                self.flatindex = self.flatindex_F
            else:
                self.flatindex = self.flatindex_C
            self.flatarray = self.node.sidsValue().flat
        else:
            self.flatarray = None
        self.hmin = 1
        self.vmin = 1
        self.font = QFont("Courier new", 10)
        fm = QFontMetrics(self.font)
        self.colwidth = fm.width(tx)

    def setRange(self, minh, minv):
        self.hmin = minh
        self.vmin = minv

    def headerData(self, section, orientation, role):
        if (orientation == Qt.Horizontal) and (role == Qt.DisplayRole):
            hix = section + self.hmin
            return hix
        if (orientation == Qt.Vertical) and (role == Qt.DisplayRole):
            vix = section + self.vmin
            return vix
        return None

    def flatindex_C(self, index):
        return index.row() * self.cs + index.column()

    def flatindex_F(self, index):
        return index.row() + index.column() * self.ls

    def columnCount(self, parent):
        return self.cs

    def rowCount(self, parent):
        return self.ls

    def index(self, row, column, parent):
        return self.createIndex(row, column, 0)

    def data(self, index, role=Qt.DisplayRole):
        if role not in [Qt.DisplayRole, Qt.FontRole]:
            return None
        if self.flatarray is None:
            return None
        if role == Qt.FontRole:
            return self.font
        return self.fmt % self.flatarray[self.flatindex(index)]

    def flags(self, index):
        if not index.isValid():
            return Qt.NoItemFlags
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable

    def getValue(self, index):
        return self.flatarray[self.flatindex(index)]

# -----------------------------------------------------------------



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CGNS/NAV/T/Q7PatternWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Q7PatternWindow(object):
    def setupUi(self, Q7PatternWindow):
        Q7PatternWindow.setObjectName("Q7PatternWindow")
        Q7PatternWindow.resize(715, 350)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Q7PatternWindow.sizePolicy().hasHeightForWidth())
        Q7PatternWindow.setSizePolicy(sizePolicy)
        Q7PatternWindow.setMinimumSize(QtCore.QSize(715, 350))
        Q7PatternWindow.setMaximumSize(QtCore.QSize(1200, 350))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/icons/cgSpy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Q7PatternWindow.setWindowIcon(icon)
        self.gridLayout = QtWidgets.QGridLayout(Q7PatternWindow)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.bInfo = QtWidgets.QPushButton(Q7PatternWindow)
        self.bInfo.setMinimumSize(QtCore.QSize(25, 25))
        self.bInfo.setMaximumSize(QtCore.QSize(25, 25))
        self.bInfo.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/icons/help-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bInfo.setIcon(icon1)
        self.bInfo.setObjectName("bInfo")
        self.horizontalLayout_2.addWidget(self.bInfo)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.bClose = QtWidgets.QPushButton(Q7PatternWindow)
        self.bClose.setObjectName("bClose")
        self.horizontalLayout_2.addWidget(self.bClose)
        self.gridLayout.addLayout(self.horizontalLayout_2, 5, 0, 1, 1)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.bAdd = QtWidgets.QPushButton(Q7PatternWindow)
        self.bAdd.setMinimumSize(QtCore.QSize(25, 25))
        self.bAdd.setMaximumSize(QtCore.QSize(25, 25))
        self.bAdd.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/icons/pattern-open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bAdd.setIcon(icon2)
        self.bAdd.setObjectName("bAdd")
        self.horizontalLayout.addWidget(self.bAdd)
        self.bDelete = QtWidgets.QPushButton(Q7PatternWindow)
        self.bDelete.setMinimumSize(QtCore.QSize(25, 25))
        self.bDelete.setMaximumSize(QtCore.QSize(25, 25))
        self.bDelete.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/icons/pattern-close.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bDelete.setIcon(icon3)
        self.bDelete.setObjectName("bDelete")
        self.horizontalLayout.addWidget(self.bDelete)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.bCopy = QtWidgets.QPushButton(Q7PatternWindow)
        self.bCopy.setMinimumSize(QtCore.QSize(25, 25))
        self.bCopy.setMaximumSize(QtCore.QSize(25, 25))
        self.bCopy.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/icons/mark-node.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bCopy.setIcon(icon4)
        self.bCopy.setObjectName("bCopy")
        self.horizontalLayout.addWidget(self.bCopy)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.bSave = QtWidgets.QPushButton(Q7PatternWindow)
        self.bSave.setMinimumSize(QtCore.QSize(25, 25))
        self.bSave.setMaximumSize(QtCore.QSize(25, 25))
        self.bSave.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/icons/pattern-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSave.setIcon(icon5)
        self.bSave.setObjectName("bSave")
        self.horizontalLayout.addWidget(self.bSave)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.patternTable = Q7PatternTableWidget(Q7PatternWindow)
        self.patternTable.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.patternTable.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.patternTable.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.patternTable.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.patternTable.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.patternTable.setColumnCount(4)
        self.patternTable.setObjectName("patternTable")
        self.patternTable.setRowCount(0)
        self.patternTable.horizontalHeader().setStretchLastSection(True)
        self.verticalLayout.addWidget(self.patternTable)
        self.gridLayout.addLayout(self.verticalLayout, 4, 0, 1, 1)

        self.retranslateUi(Q7PatternWindow)
        QtCore.QMetaObject.connectSlotsByName(Q7PatternWindow)

    def retranslateUi(self, Q7PatternWindow):
        _translate = QtCore.QCoreApplication.translate
        Q7PatternWindow.setWindowTitle(_translate("Q7PatternWindow", "Form"))
        self.bClose.setText(_translate("Q7PatternWindow", "Close"))
        self.patternTable.setSortingEnabled(True)
from CGNS.NAV.mpattern import Q7PatternTableWidget
from . import Res_rc
